@@ -554,13 +554,75 @@ axis_post_add (GogObject *axis, GogAxisType t)
 }
 
 static void
-axis_pre_remove (GogObject *parent, GogObject *axis)
+axis_pre_remove (GogObject *parent, GogObject *child)
 {
 	GogChart *chart = GOG_CHART (parent);
+	GogAxis *axis = GOG_AXIS (child);
+	GogColorScale *scale = gog_axis_get_color_scale (axis);
+	if (scale)
+		gog_color_scale_set_axis (scale, NULL);
 	gog_axis_clear_contributors (GOG_AXIS (axis));
 	chart->axes = g_slist_remove (chart->axes, axis);
 }
 
+/*  Color scale related code */
+static gboolean
+color_scale_can_add (GogObject const *parent)
+{
+	/* TRUE if there are more color or pseudo-3d axes than there are color scales */
+	GogChart *chart = GOG_CHART (parent);
+	GSList *ptr;
+	GogAxis *axis;
+	GogAxisType type;
+
+	if ((chart->axis_set & ((1 << GOG_AXIS_COLOR) | (1 << GOG_AXIS_PSEUDO_3D ))) == 0)
+		return FALSE;
+	for (ptr = chart->axes; ptr && ptr->data; ptr = ptr->next) {
+		axis = GOG_AXIS (ptr->data);
+		type = gog_axis_get_atype (axis);
+		if ((type == GOG_AXIS_COLOR || type == GOG_AXIS_PSEUDO_3D)
+		    && gog_axis_get_color_scale (axis) == NULL)
+			return TRUE;
+	}
+	return FALSE;
+}
+
+static void
+color_scale_post_add (GogObject *parent, GogObject *child)
+{
+	/* Link the color scale to an axis without a preexisting color scale */
+	GogChart *chart = GOG_CHART (parent);
+	GSList *ptr;
+	GogAxis *axis;
+	GogAxisType type;
+	GSList const *l;
+
+	for (ptr = chart->axes; ptr && ptr->data; ptr = ptr->next) {
+		axis = GOG_AXIS (ptr->data);
+		type = gog_axis_get_atype (axis);
+		if ((type == GOG_AXIS_COLOR || type == GOG_AXIS_PSEUDO_3D)
+		    && gog_axis_get_color_scale (axis) == NULL) {
+				gog_color_scale_set_axis (GOG_COLOR_SCALE (child), axis);
+				for (l = gog_axis_contributors (axis); l; l = l->next)
+					gog_object_request_update (GOG_OBJECT (l->data));
+				break;
+			}
+	}
+	gog_chart_request_cardinality_update (chart);
+}
+
+static void
+color_scale_pre_remove (GogObject *parent, GogObject *scale)
+{
+	/* Unlink the color scale */
+	GSList const *l = gog_axis_contributors (gog_color_scale_get_axis (GOG_COLOR_SCALE (scale)));
+	gog_color_scale_set_axis (GOG_COLOR_SCALE (scale), NULL);
+	for (; l; l = l->next)
+		gog_object_request_update (GOG_OBJECT (l->data));
+	gog_chart_request_cardinality_update (GOG_CHART (parent));
+}
+
+
 static gboolean x_axis_can_add (GogObject const *parent) { return axis_can_add (parent, GOG_AXIS_X); }
 static void x_axis_post_add    (GogObject *parent, GogObject *child)  { axis_post_add   (child, GOG_AXIS_X); }
 static gboolean y_axis_can_add (GogObject const *parent) { return axis_can_add (parent, GOG_AXIS_Y); }
@@ -650,7 +712,13 @@ static GogObjectRole const roles[] = {
 #endif
 	{ N_("3D-Box"), "Gog3DBox",	1,
 	  GOG_POSITION_SPECIAL, GOG_POSITION_SPECIAL, GOG_OBJECT_NAME_BY_ROLE,
-	  role_3d_box_can_add, role_3d_box_can_remove, NULL, NULL, NULL, NULL, { -1 } }
+	  role_3d_box_can_add, role_3d_box_can_remove, NULL, NULL, NULL, NULL, { -1 } },
+	{ N_("Color-Scale"), "GogColorScale", 7,
+	  GOG_POSITION_COMPASS|GOG_POSITION_ANY_MANUAL|GOG_POSITION_ANY_MANUAL_SIZE,
+	  GOG_POSITION_E|GOG_POSITION_ALIGN_CENTER,
+	  GOG_OBJECT_NAME_BY_ROLE,
+	  color_scale_can_add, NULL, NULL,
+	  color_scale_post_add, color_scale_pre_remove, NULL, { -1 } }
 };
 
 static GogManualSizeMode
